# BasePermission

权限工具类

## 使用方法

### 如果你的项目 Gradle 配置是在 7.0 以下，需要在 build.gradle 文件中加入

~~~
repositories {
        maven {
            url "http://maven.keleyanyi.com/repository/benyanyi/"
        }
    }
~~~

### 如果你的 Gradle 配置是 7.0 及以上，则需要在 settings.gradle 文件中加入
~~~
dependencyResolutionManagement {
    repositories {
        maven {
            allowInsecureProtocol = true
            url "http://maven.keleyanyi.com/repository/benyanyi/"
        }
    }
}
~~~

### 在需要权限请求模块下的 build.gradle 文件中加入远程依赖

~~~
implementation 'com.yanyi.benyanyi:PermissionHelper:1.1.6'
~~~

### 使用

~~~
        PermissionHelper.with(this)
                .setPermissions(Manifest.permission.READ_PHONE_STATE,
                        Manifest.permission.CALL_PHONE,
                        Manifest.permission.ACCESS_COARSE_LOCATION,
                        Manifest.permission.ACCESS_FINE_LOCATION)
                .setDefaultPermissionDialogInfo()
                .request(new PermissionCallBack() {
                    @Override
                    public void onPermissionSuccess() {
                        log("所有权限请求成功");
                    }

                    @Override
                    public void onPermissionFailure(String[] failurePermissions) {
                        log("部分权限请求失败");
                    }

                    @Override
                    public void onPermissionComplete() {
                        log("请求结束");
                    }
                });
~~~

### 版本更新

* 2022-08-31更新(1.1.6) 修改方法，去除多余方法及多余配置项，添加默认dialog配置,优化跳转设置页设置权限后返回权限请求结果无反应，适配android11所有文件访问权限
* 2021-08-27更新(1.1.4) 删除kotlin版本，升级gradle版本，修改maven地址
* 2021-03-23更新(1.1.3) 添加kotlin版1.1.3版本
* 2021-03-12更新(1.1.3) 修复不同页面调用权限请求时，activity为空报错
* 2021-02-27更新(1.1.2) 添加销毁方法，防止请求未销毁造成的报错
* 2021-02-03更新(1.1.1) 将接收FragmentActivity更改为Activity
* 2020-05-18更新(1.1.0) 将android support转成androidx支持,并将Maven库存放到个人服务器上
* 2019-07-18更新(1.0.9) 优化拒绝权限通知弹窗，添加注解配置弹窗方法，优化注解方法，去除多余配置
* 2019-06-26更新(1.0.8) 删除重复方法
* 2019-06-12更新(1.0.7) 优化方法，规范方法，修改包名，以便所有开源库统一包名
* 2019-05-30更新(1.0.6) 优化注解调用方法
* 2019-05-24更新(1.0.5) 添加注解调用申请权限
* 2019-01-21更新(1.0.2) 修复自定义弹窗信息，弹窗设置不显示状态时没回调
* 2018-06-09更新(1.0.1) 新增弹窗配置信息修改(PermissionDialogInfo)
* 2018-06-08创建(1.0.0) 创建